import clr
import sys
from typing import Dict
from time import sleep

from System import Decimal
from System import Action
from System import UInt64
from System import UInt32

kinesis_path = 'C:\\Program Files\\Thorlabs\\Kinesis'
sys.path.append(kinesis_path)

clr.AddReference("Thorlabs.MotionControl.DeviceManagerCLI")
clr.AddReference("Thorlabs.MotionControl.IntegratedStepperMotorsCLI")
clr.AddReference("Thorlabs.MotionControl.GenericMotorCLI")
clr.AddReference("Thorlabs.MotionControl.IntegratedStepperMotorsCLI")
clr.AddReference("Thorlabs.MotionControl.DeviceManagerCLI")
clr.AddReference("Thorlabs.MotionControl.GenericMotorCLI")
clr.AddReference("Thorlabs.MotionControl.FilterFlipperCLI")
clr.AddReference("Thorlabs.MotionControl.Benchtop.BrushlessMotorCLI")
clr.AddReference("Thorlabs.MotionControl.KCube.PiezoCLI")
clr.AddReference("Thorlabs.MotionControl.TCube.DCServoCLI ")
clr.AddReference("Thorlabs.MotionControl.KCube.DCServoCLI ")

import Thorlabs.MotionControl.FilterFlipperCLI as FilterFlipper
import Thorlabs.MotionControl.IntegratedStepperMotorsCLI as Integrated
import Thorlabs.MotionControl.DeviceManagerCLI as Device
import Thorlabs.MotionControl.GenericMotorCLI as Generic
import Thorlabs.MotionControl.Benchtop.BrushlessMotorCLI as BrushlessMotorCLI
import Thorlabs.MotionControl.KCube.PiezoCLI as KCubePiezo
import Thorlabs.MotionControl.TCube.DCServoCLI as TCubeDCServo
import Thorlabs.MotionControl.KCube.DCServoCLI as KCubeDCServo


# First, build device list
Device.DeviceManagerCLI.BuildDeviceList()

# Then, get serial numbers for each category
_sn_list = lambda prefix: [str(sn) for sn in Device.DeviceManagerCLI.GetDeviceList(prefix)]
serialnumbers_integrated_stepper = _sn_list(Integrated.CageRotator.DevicePrefix)
serialnumbers_flipper = _sn_list(FilterFlipper.FilterFlipper.DevicePrefix)
serialnumbers_brushless = _sn_list(BrushlessMotorCLI.BenchtopBrushlessMotor.DevicePrefix)
serialnumbers_piezo = _sn_list(KCubePiezo.KCubePiezo.DevicePrefix)
serialnumbers_tcube_dcservo = _sn_list(TCubeDCServo.TCubeDCServo.DevicePrefix)
serialnumbers_kcube_dcservo = _sn_list(KCubeDCServo.KCubeDCServo.DevicePrefix)


class Kinesis:
    default_units = ''

    def __init__(self):
        self._device = None

    def connect(self, serial: int):
        self._device.Connect(serial)
        self._device.WaitForSettingsInitialized(5000)
        self._device.StartPolling(250)

    def close(self):
        """
            close the current instance of Kinesis instrument.
        """
        self._device.StopPolling()
        self._device.Disconnect()
        self._device.Dispose()
        self._device = None

    @property
    def name(self) -> str:
        return self._device.GetDeviceInfo().Name

    @property
    def serial_number(self) -> str:
        return self._device.GetDeviceInfo().SerialNumber

    @property
    def backlash(self):
        return Decimal.ToDouble(self._device.GetBacklash())

    @backlash.setter
    def backlash(self, backlash: float):
        self._device.SetBacklash(Decimal(backlash))

    def stop(self):
        self._device.Stop(0)

    def move_done_callback(self, val: int):
        print('move done')

    def move_abs(self, position: float, callback=None, **kwargs):
        if callback is not None:
            callback = Action[UInt64](callback)
        else:
            callback = 0
        self._device.MoveTo(Decimal(position), callback)

    def move_rel(self, position: float, callback=None, **kwargs):
        if callback is not None:
            callback = Action[UInt64](callback)
        else:
            callback = 0
        self._device.MoveRelative(Generic.MotorDirection.Forward, Decimal(position), callback)

    def home(self, callback=None):
        if callback is not None:
            callback = Action[UInt64](callback)
        else:
            callback = 0
        self._device.Home(callback)

    @property
    def is_homed(self) -> bool:
        return self._device.Status.IsHomed

    @property
    def is_moving(self) -> bool:
        return self._device.Status.IsInMotion

    @property
    def is_homing(self) -> bool:
        return self._device.Status.IsHoming

    def get_position(self, **kwargs):
        raise NotImplementedError

    def get_target_position(self, *args, **kwargs) -> float:
        return Decimal.ToDouble(self._device.Position)

    def get_units(self, *args, **kwargs) -> str:
        """ Get the stage units from the controller

        """
        try:
            units = self._device.get_UnitConverter().RealUnits
        except Exception:
            units = self.default_units
        return units


class IntegratedStepper(Kinesis):
    """ Specific Kinesis class for Integrated Stepper motor"""

    default_units = '°'

    def __init__(self):
        super().__init__()
        self._device: Integrated.CageRotator = None

    def connect(self, serial: int):
        if serial in serialnumbers_integrated_stepper:
            self._device = Integrated.CageRotator.CreateCageRotator(serial)
            super().connect(serial)
            if not (self._device.IsSettingsInitialized()):
                raise (Exception("no Stage Connected"))
            else:
                self._device.LoadMotorConfiguration(serial)
        else:
            raise ValueError('Invalid Serial Number')

    def get_position(self, **kwargs):
        return Decimal.ToDouble(self._device.ContinuousRotationPosition)

    def get_units(self, *args, **kwargs) -> str:
        return super().get_units()


class BrushlessMotorChannel(Kinesis):
    properties = ['MaxPosition', 'MinPosition',
                  'MaxAcceleration', 'MaxDecceleration',
                  'MaxVelocity']

    default_units = 'mm'

    def __init__(self, controller_device: BrushlessMotorCLI.BenchtopBrushlessMotor,
                 channel_index=1):
        super().__init__()
        self._controller = controller_device
        self._device: BrushlessMotorCLI.BrushlessMotorChannel = None
        self._channel_index = channel_index
        self.motorConfiguration = None

    def get_property(self, prop: str):
        if prop in self.properties:
            return Decimal.ToDouble(getattr(self._device.GetStageAxisParams(), prop))

    def set_property(self, prop: str, value: float):
        if prop in self.properties:
            return setattr(self._device.GetStageAxisParams(), prop, Decimal(value))

    def connect(self, *args, **kwargs):
        self._device: BrushlessMotorCLI.BrushlessMotorChannel = (
            self._controller.GetChannel(self._channel_index))
        self._device.WaitForSettingsInitialized(5000)

        if not (self._device.IsSettingsInitialized()):
            raise (Exception("no Stage Connected"))
        else:
            self.motorConfiguration = self._device.LoadMotorConfiguration(self._device.DeviceID)
        self._device.StartPolling(250)
        self._device.EnableDevice()

    def get_position(self) -> float:
        return Decimal.ToDouble(self._device.get_DevicePosition())


class BrushlessDCMotor(Kinesis):
    """ Specific Kinesis class for Brushless DC Motors"""
    n_channels = 3
    default_units = 'mm'

    def __init__(self):
        super().__init__()
        self._device: BrushlessMotorCLI.BenchtopBrushlessMotor = None
        self._channels: Dict[int, BrushlessMotorChannel] = {}
        self._current_channel_index = 1

    def connect(self, serial: int):
        if serial in serialnumbers_brushless:
            self._device = (
                BrushlessMotorCLI.BenchtopBrushlessMotor.CreateBenchtopBrushlessMotor(serial))
            self._device.Connect(serial)

    def init_channel(self, channel=1) -> BrushlessMotorChannel:
        self._channels[channel] = BrushlessMotorChannel(self._device, channel)
        self._channels[channel].connect()
        return self._channels[channel]

    def get_position(self, channel: int = 1) -> float:
        if channel not in self._channels:
            self.init_channel(channel)
        return self._channels[channel].get_position()

    def move_abs(self, position: float, callback=None, channel: int = 1):
        if channel not in self._channels:
            self.init_channel(channel)
        self._channels[channel].move_abs(position, callback)

    def close(self):
        for channel in self._channels.values():
            channel.close()
        self._device.Disconnect()

    def home(self, channel: int = 1, callback=None):
        if channel not in self._channels:
            self.init_channel(channel)
        self._channels[channel].home(callback)

    def is_homed(self, channel: int = 1) -> bool:
        if channel not in self._channels:
            self.init_channel(channel)
        return self._channels[channel].is_homed

    def stop(self, channel: int = 1):
        if channel not in self._channels:
            self.init_channel(channel)
        self._channels[channel].stop()

    def get_units(self, channel: int = 1) -> str:
        """ Get the stage units from the controller
        """
        if channel not in self._channels:
            self.init_channel(channel)
        return self._channels[channel].get_units()

    def get_target_position(self, channel: int = 1) -> float:
        if channel not in self._channels:
            self.init_channel(channel)
        return self._channels[channel].get_target_position()


class Flipper(Kinesis):
    """ Specific Kinesis class for Flipper"""

    default_units = ''

    def __init__(self):
        super().__init__()
        self._device: FilterFlipper.FilterFlipper = None

    def connect(self, serial: int):
        if serial in serialnumbers_flipper:
            self._device = FilterFlipper.FilterFlipper.CreateFilterFlipper(serial)
            super().connect(serial)
            if not (self._device.IsSettingsInitialized()):
                raise (Exception("no Stage Connected"))
        else:
            raise ValueError('Invalid Serial Number')

    def move_abs(self, position: float, callback=None, **kwargs):
        if int(position) == 1:
            position = 1
        else:
            position = 2
        self._device.SetPosition(UInt32(position), 0)

    def get_position(self, **kwargs):
        position = int(self._device.Position)
        if position == 2:
            position = 0
        else:
            position = 1
        return position


class Piezo(Kinesis):
    default_units = 'V'

    def __init__(self):
        self._device: KCubePiezo.KCubePiezo = None

    def connect(self, serial: int):
        if serial in serialnumbers_piezo:
            self._device = (
                KCubePiezo.KCubePiezo.CreateKCubePiezo(serial))
            self._device.Connect(serial)
            self._device.StartPolling(250)
            self._device.EnableDevice()
            self._device.GetPiezoConfiguration(serial)
        else:
            raise ValueError('Invalid Serial Number')

    def move_abs(self, position: float):
        self._device.SetOutputVoltage(Decimal(position))

    def home(self):
        self.move_abs(0.0)

    def get_position(self) -> float:
        return Decimal.ToDouble(self._device.GetOutputVoltage())

    def stop(self):
        pass


class DCServoTCube(Kinesis):
    """ Specific Kinesis class for Brushless DC Motors"""
    n_channels = 1
    default_units = 'mm'

    def __init__(self):
        super().__init__()
        self._device: TCubeDCServo.TCubeDCServo = None

    def connect(self, serial: int):
        if serial in serialnumbers_tcube_dcservo:
            self._device = (
                TCubeDCServo.TCubeDCServo.CreateTCubeDCServo(serial))
            super().connect(serial)
            self._device.EnableDevice()
            sleep(0.5)
            self._device.LoadMotorConfiguration(serial)
            self.motor_settings = self._device.MotorDeviceSettings


    def get_position(self) -> float:
        return Decimal.ToDouble(self._device.get_DevicePosition())


class DCServoKCube(Kinesis):
    """ Specific Kinesis class for KCube controllers"""
    n_channels = 1
    default_units = 'mm'

    def __init__(self):
        super().__init__()
        self._device: KCubeDCServo.KCubeDCServo = None

    def connect(self, serial: int):
        if serial in serialnumbers_kcube_dcservo:
            self._device = (
                KCubeDCServo.KCubeDCServo.CreateKCubeDCServo(serial))
            super().connect(serial)
            self._device.EnableDevice()
            sleep(0.5)
            self._device.LoadMotorConfiguration(serial)
            self.motor_settings = self._device.MotorDeviceSettings


    def get_position(self) -> float:
        return Decimal.ToDouble(self._device.get_DevicePosition())


if __name__ == '__main__':
    if False:
        controller = BrushlessDCMotor()
        controller.connect(serialnumbers_brushless[0])
        motor = controller.init_channel(1)
        print(motor.get_units())
        motor.home()
        print(f'homing: {motor.is_homing}')
        print(f'Moving: {motor.is_moving}')
        print(motor.get_target_position())

        motor.move_abs(87, motor.move_done_callback)
        print(f'homing: {motor.is_homing}')
        print(f'Moving: {motor.is_moving}')
        print(motor.get_target_position())

        controller.close()

    elif True:
        controller = DCServoTCube()
        controller.connect(serialnumbers_tcube_dcservo[0])

        controller.close()



import numpy as np

from pymodaq.utils.data import DataFromPlugins, Axis, DataToExport
from pymodaq.control_modules.viewer_utility_classes import DAQ_Viewer_base, comon_parameters, main


from pymodaq_utils.logger import set_logger, get_module_name
from pymodaq_utils.utils import ThreadCommand
from pymodaq_gui.parameter import Parameter

from pymodaq_plugins_thorlabs.hardware.ccsxxx import CCSXXX

class DAQ_1DViewer_CCSXXX(DAQ_Viewer_base):
    """ Instrument plugin class for a 1D viewer.

    This object inherits all functionalities to communicate with PyMoDAQ’s DAQ_Viewer module through inheritance via
    DAQ_Viewer_base. It makes a bridge between the DAQ_Viewer module and the Python wrapper of a particular instrument.

    Attributes:
    -----------
    controller: object
        The particular object that allow the communication with the hardware, in general a python wrapper around the
         hardware library.

    """
    params = comon_parameters + [
        {'title': 'Integration time', 'name': 'integration_time', 'type': 'float', 'value': 100.0e-3}, # in seconds
        {'title': 'Resource name', 'name': 'resource_name', 'type': 'str', 'value': 'USB0::0x1313::0x8087::M00934802::RAW'},
    ]

    def ini_attributes(self):
        """Initialize attributes for the DAQ_1DViewer_CCSXXX class."""
        self.controller: CCSXXX = None
        self.x_axis = None

    def commit_settings(self, param: Parameter):
        """Apply the consequences of a change of value in the detector settings

        Parameters
        ----------
        param: Parameter
            A given parameter (within detector_settings) whose value has been changed by the user
        """
        if param.name() == "integration_time":
            self.controller.set_integration_time(self.settings['integration_time'])

    def ini_detector(self, controller=None):
        """Detector communication initialization

        Parameters
        ----------
        controller: (object)
            custom object of a PyMoDAQ plugin (Slave case). None if only one actuator/detector by controller
            (Master case)

        Returns
        -------
        info: str
        initialized: bool
            False if initialization failed otherwise True
        """
        self.ini_detector_init(slave_controller=controller)

        if self.is_master:
            self.controller = CCSXXX(self.settings['resource_name'])
            self.controller.connect()

        data_x_axis = self.controller.get_wavelength_data()
        self.x_axis = Axis(data=data_x_axis, label='Wavelength', units='nm', index=0)

        self.dte_signal_temp.emit(DataToExport(name='CCSXXX',
                                               data=[DataFromPlugins(name='Spectrum',
                                                                     data=[np.zeros(len(data_x_axis)),],
                                                                     dim='Data1D', labels=['Intensity'],
                                                                     axes=[self.x_axis])]))

        info = "CCSXXX spectrometer initialized"
        initialized = True
        return info, initialized

    def close(self):
        """Terminate the communication protocol"""
        self.controller.close()

    def grab_data(self, Naverage=1, **kwargs):
        """Start a grab from the detector

        Parameters
        ----------
        Naverage: int
            Number of hardware averaging (if hardware averaging is possible, self.hardware_averaging should be set to
            True in class preamble and you should code this implementation)
        kwargs: dict
            others optionals arguments
        """
        self.controller.start_scan()
        data_tot = self.controller.get_scan_data()
        self.dte_signal.emit(DataToExport('CCSXXX',
                                          data=[DataFromPlugins(name='Spectrum', data=[data_tot],
                                                                dim='Data1D', labels=['Intensity'],
                                                                axes=[self.x_axis])]))

    def stop(self):
        """Stop the current grab hardware wise if necessary"""
        # raise NotImplemented  # when writing your own plugin remove this line
        return ''

if __name__ == '__main__':
    main(__file__)



from pymodaq.control_modules.move_utility_classes import (
    DAQ_Move_base, comon_parameters_fun, main, DataActuatorType, DataActuator)
from pymodaq_utils.utils import ThreadCommand

from pymodaq_gui.parameter import Parameter

from pymodaq_utils.logger import set_logger, get_module_name

logger = set_logger(get_module_name(__file__))


class DAQ_Move_DCServoCube_Abstract(DAQ_Move_base):
    """ Abstract class for Thorlabs DCServo cube instrument plugin.

    This object inherits all functionalities to communicate with PyMoDAQ’s DAQ_Move module through inheritance via
    DAQ_Move_base. It makes a bridge between the DAQ_Move module and the Python wrapper of a particular instrument.

    Attributes:
    -----------
    controller: object
        The particular object that allow the communication with the hardware, in general a python wrapper around the
         hardware library.

    """

    # Type of controller used.
    # To redefine in the child class.
    controller_type = None

    # Child class-specific parameters go here.
    # Example:
    #
    #is_multiaxes = False
    #_axes_names = ['']
    #_epsilon = 0.005
    #data_actuator_type = DataActuatorType.DataActuator
    #params = [
    #             {'title': 'Serial Number:', 'name': 'serial_number', 'type': 'list',
    #              'limits': serialnumbers_tcube_dcservo, 'value': serialnumbers_tcube_dcservo[0]}
    #
    #         ] + comon_parameters_fun(is_multiaxes, axes_names=_axes_names, epsilon=_epsilon)

    def ini_attributes(self):
        self.controller: self.controller_type = None
        self._move_done = False


    def move_done_callback(self, val: int):
        """ will be triggered for each end of move: abs, rel or homing"""
        self._move_done = True
        self.stop_motion()
        logger.debug('Callback called')


    def user_condition_to_reach_target(self) -> bool:
        """ Implement a condition for exiting the polling mechanism and specifying that the
        target value has been reached

       Returns
        -------
        bool: if True, PyMoDAQ considers the target value has been reached
        """
        return self._move_done


    def get_actuator_value(self):
        """Get the current value from the hardware with scaling conversion.

        Returns
        -------
        float: The position obtained after scaling conversion.
        """
        pos = DataActuator(
            data=self.controller.get_position(),
            units=self.controller.get_units()
        )
        pos = self.get_position_with_scaling(pos)
        return pos


    def close(self):
        """Terminate the communication protocol"""
        if self.is_master:
            self.controller.close()


    def commit_settings(self, param: Parameter):
        """Apply the consequences of a change of value in the detector settings

        Parameters
        ----------
        param: Parameter
            A given parameter (within detector_settings) whose value has been changed by the user
        """
        pass


    def ini_stage(self, controller=None):
        """Actuator communication initialization

        Parameters
        ----------
        controller: (object)
            custom object of a PyMoDAQ plugin (Slave case). None if only one actuator by controller (Master case)

        Returns
        -------
        info: str
        initialized: bool
            False if initialization failed otherwise True
        """

        if self.is_master:
            self.controller = self.controller_type()
            self.controller.connect(self.settings['serial_number'])
        else:
            self.controller = controller

        # update the axis unit by interogating the controller and the specific axis
        self.axis_unit = self.controller.get_units()

        if not self.controller.is_homed:
            self.move_home()

        info = f"{self.controller.name} - {self.controller.serial_number}"
        initialized = True
        return info, initialized


    def move_abs(self, value: DataActuator):
        """ Move the actuator to the absolute target defined by value

        Parameters
        ----------
        value: (float) value of the absolute target positioning
        """
        self._move_done = False
        value = self.check_bound(value)
        self.target_value = value
        value = self.set_position_with_scaling(value)  # apply scaling if the user specified one
        self.controller.move_abs(value.value(), callback=self.move_done_callback)


    def move_rel(self, value: DataActuator):
        """ Move the actuator to the relative target actuator value defined by value

        Parameters
        ----------
        value: (float) value of the relative target positioning
        """
        self._move_done = False
        value = self.check_bound(self.current_position + value) - self.current_position
        self.target_value = value + self.current_position
        value = self.set_position_relative_with_scaling(value)
        self.controller.move_rel(value.value(), callback=self.move_done_callback)


    def move_home(self):
        """Call the reference method of the controller"""
        self._move_done = False
        self.controller.home(callback=self.move_done_callback)


    def stop_motion(self):
        """Stop the actuator and emits move_done signal"""
        self.controller.stop()



from pathlib import Path
from pymodaq_utils.logger import set_logger  # to be imported by other modules.
from pymodaq_utils.utils import get_version, PackageNotFoundError

from .utils import Config
config = Config()

try:
    __version__ = get_version(__package__)
except PackageNotFoundError:
    __version__ = '0.0.0dev'


from pymodaq.control_modules.move_utility_classes import (
    DAQ_Move_base, comon_parameters_fun, main, DataActuatorType, DataActuator)

from pymodaq_utils.logger import set_logger, get_module_name
from pymodaq_utils.utils import ThreadCommand
from pymodaq_gui.parameter import Parameter

from pymodaq_plugins_thorlabs.hardware.daq_move_servocube_abstract import DAQ_Move_DCServoCube_Abstract
from pymodaq_plugins_thorlabs.hardware.kinesis import DCServoTCube, serialnumbers_tcube_dcservo


logger = set_logger(get_module_name(__file__))


class DAQ_Move_DCServoTCube(DAQ_Move_DCServoCube_Abstract):
    """ Instrument plugin class for Thorlabs DC Servo TCube.

    This object inherits all functionalities to communicate with PyMoDAQ’s DAQ_Move module through inheritance via
    DAQ_Move_base. It makes a bridge between the DAQ_Move module and the Python wrapper of a particular instrument.

    Attributes:
    -----------
    controller: object
        The particular object that allow the communication with the hardware, in general a python wrapper around the
         hardware library.

    """

    # Controller type
    controller_type = DCServoTCube

    # Parameters
    _controller_units = DCServoTCube.default_units
    is_multiaxes = False
    _axes_names = ['']
    _epsilon = 0.005
    data_actuator_type = DataActuatorType.DataActuator
    params = [
                 {'title': 'Serial Number:', 'name': 'serial_number', 'type': 'list',
                  'limits': serialnumbers_tcube_dcservo, 'value': serialnumbers_tcube_dcservo[0]}

             ] + comon_parameters_fun(is_multiaxes, axes_names=_axes_names, epsilon=_epsilon)


if __name__ == '__main__':
    main(__file__, init=False)



from pymodaq.control_modules.move_utility_classes import (
    DAQ_Move_base, comon_parameters_fun, main, DataActuatorType, DataActuator)

from pymodaq_utils.logger import set_logger, get_module_name
from pymodaq_utils.utils import ThreadCommand
from pymodaq_gui.parameter import Parameter

from pymodaq_plugins_thorlabs.hardware.kinesis import serialnumbers_piezo, Piezo


logger = set_logger(get_module_name(__file__))


class DAQ_Move_KPZ101(DAQ_Move_base):
    """ Instrument plugin class for an actuator.

    This object inherits all functionalities to communicate with PyMoDAQ’s DAQ_Move module through inheritance via
    DAQ_Move_base. It makes a bridge between the DAQ_Move module and the Python wrapper of a particular instrument.

    Attributes:
    -----------
    controller: object
        The particular object that allow the communication with the hardware, in general a python wrapper around the
         hardware library.

    """
    _controller_units = Piezo.default_units
    is_multiaxes = False
    _axes_names = {'1': 1}
    _epsilon = 0.01
    data_actuator_type = DataActuatorType.DataActuator
    params = [
                 {'title': 'Serial Number:', 'name': 'serial_number', 'type': 'list',
                  'limits': serialnumbers_piezo, 'value': serialnumbers_piezo[0]},
                  {'title': 'Units:', 'name': 'units', 'type': 'string', 'value': _controller_units}

             ] + comon_parameters_fun(is_multiaxes, axes_names=_axes_names, epsilon=_epsilon)

    def ini_attributes(self):
        self.controller: Piezo = None

    def get_actuator_value(self):
        """Get the current value from the hardware with scaling conversion.

        Returns
        -------
        DataActuator: The position obtained after scaling conversion.
        """
        pos = DataActuator(
            data=self.controller.get_position(),
            units=self.controller.get_units()
        )
        pos = self.get_position_with_scaling(pos)
        return pos

    def close(self):
        """Terminate the communication protocol"""
        if self.is_master:
            self.controller.close()

    def commit_settings(self, param: Parameter):
        """Apply the consequences of a change of value in the detector settings

        Parameters
        ----------
        param: Parameter
            A given parameter (within detector_settings) whose value has been changed by the user
        """
        if param.name() == 'units':
            self.axis_unit = self.controller.get_units(self.axis_value)

    def ini_stage(self, controller=None):
        """Actuator communication initialization

        Parameters
        ----------
        controller: (object)
            custom object of a PyMoDAQ plugin (Slave case). None if only one actuator by controller (Master case)

        Returns
        -------
        info: str
        initialized: bool
            False if initialization failed otherwise True
        """

        if self.is_master:
            self.controller = Piezo()
            self.controller.connect(self.settings['serial_number'])
        else:
            self.controller = controller

        self.axis_unit = self._controller_units

        info = f'{self.controller.name} - {self.controller.serial_number}'
        initialized = True
        return info, initialized

    def move_abs(self, value: DataActuator):
        """ Move the actuator to the absolute target defined by value

        Parameters
        ----------
        value: (DataActuator) value of the absolute target positioning
        """
        value = self.check_bound(value)
        self.target_value = value
        value = self.set_position_with_scaling(value) 
        self.controller.move_abs(value.value())

    def move_rel(self, value: DataActuator):
        """ Move the actuator to the relative target actuator value defined by value

        Parameters
        ----------
        value: (DataActuator) value of the relative target positioning
        """
        value = self.check_bound(self.current_value + value) - self.current_value
        self.target_value = value + self.current_value
        value = self.set_position_with_scaling(self.target_value)
        self.controller.move_abs(value.value())

    def move_home(self):
        """Call the reference method of the controller"""
        self.controller.home()

    def stop_motion(self):
        """Stop the actuator and emits move_done signal"""
        self.controller.stop()


if __name__ == '__main__':
    main(__file__, init=False)



from pymodaq.control_modules.move_utility_classes import (
    DAQ_Move_base, comon_parameters_fun, main, DataActuatorType, DataActuator)

from pymodaq_utils.logger import set_logger, get_module_name
from pymodaq_utils.utils import ThreadCommand
from pymodaq_gui.parameter import Parameter

from pymodaq_plugins_thorlabs.hardware.kinesis import BrushlessDCMotor, serialnumbers_brushless

logger = set_logger(get_module_name(__file__))


class DAQ_Move_BrushlessDCMotor(DAQ_Move_base):
    """ Instrument plugin class for an actuator.

    This object inherits all functionalities to communicate with PyMoDAQ’s DAQ_Move module through inheritance via
    DAQ_Move_base. It makes a bridge between the DAQ_Move module and the Python wrapper of a particular instrument.

    Attributes:
    -----------
    controller: object
        The particular object that allow the communication with the hardware, in general a python wrapper around the
         hardware library.

    """
    _controller_units = BrushlessDCMotor.default_units
    is_multiaxes = True
    _axes_names = {'1': 1, '2': 2, '3': 3}
    _epsilon = 0.01
    data_actuator_type = DataActuatorType.DataActuator
    params = [
                 {'title': 'Serial Number:', 'name': 'serial_number', 'type': 'list',
                  'limits': serialnumbers_brushless, 'value': serialnumbers_brushless[0]}

             ] + comon_parameters_fun(is_multiaxes, axes_names=_axes_names, epsilon=_epsilon)

    def ini_attributes(self):
        self.controller: BrushlessDCMotor = None
        self._move_done = False

    def move_done_callback(self, val: int):
        """ will be triggered for each end of move: abs, rel or homing"""
        self._move_done = True
        self.stop_motion()
        logger.debug('Callback called')

    def user_condition_to_reach_target(self) -> bool:
        """ Implement a condition for exiting the polling mechanism and specifying that the
        target value has been reached

       Returns
        -------
        bool: if True, PyMoDAQ considers the target value has been reached
        """

        return self._move_done

    def get_actuator_value(self):
        """Get the current value from the hardware with scaling conversion.

        Returns
        -------
        float: The position obtained after scaling conversion.
        """
        pos = DataActuator(
            data=self.controller.get_position(self.axis_value),
            units=self.controller.get_units(self.axis_value)
        )
        pos = self.get_position_with_scaling(pos)
        return pos

    def close(self):
        """Terminate the communication protocol"""
        if self.is_master:
            self.controller.close()

    def commit_settings(self, param: Parameter):
        """Apply the consequences of a change of value in the detector settings

        Parameters
        ----------
        param: Parameter
            A given parameter (within detector_settings) whose value has been changed by the user
        """
        if param.name() == 'axis':
            self.axis_unit = self.controller.get_units(self.axis_value)
            # update the units are they are not known before hand in the driver class but only
            # after initialization of the controller

    def ini_stage(self, controller=None):
        """Actuator communication initialization

        Parameters
        ----------
        controller: (object)
            custom object of a PyMoDAQ plugin (Slave case). None if only one actuator by controller (Master case)

        Returns
        -------
        info: str
        initialized: bool
            False if initialization failed otherwise True
        """

        if self.is_master:
            self.controller = BrushlessDCMotor()
            self.controller.connect(self.settings['serial_number'])
        else:
            self.controller = controller

        # update the axis unit by interogating the controller and the specific axis
        self.axis_unit = self.controller.get_units(self.axis_value)

        if not self.controller.is_homed(self.axis_value):
            self.move_home()

        info = f'{self.controller.name} - {self.controller.serial_number}'
        initialized = True
        return info, initialized

    def move_abs(self, value: DataActuator):
        """ Move the actuator to the absolute target defined by value

        Parameters
        ----------
        value: (float) value of the absolute target positioning
        """
        self._move_done = False
        value = self.check_bound(value)
        self.target_value = value
        value = self.set_position_with_scaling(value)  # apply scaling if the user specified one
        self.controller.move_abs(value.value(), channel=self.axis_value,
                                 callback=self.move_done_callback)

    def move_rel(self, value: DataActuator):
        """ Move the actuator to the relative target actuator value defined by value

        Parameters
        ----------
        value: (float) value of the relative target positioning
        """
        self._move_done = False
        value = self.check_bound(self.current_position + value) - self.current_position
        self.target_value = value + self.current_position
        value = self.set_position_relative_with_scaling(value)
        self.controller.move_abs(self.target_value.value(), channel=self.axis_value,
                                 callback=self.move_done_callback)

    def move_home(self):
        """Call the reference method of the controller"""
        self._move_done = False
        self.controller.home(channel=self.axis_value, callback=self.move_done_callback)

    def stop_motion(self):
        """Stop the actuator and emits move_done signal"""

        self.controller.stop(self.axis_value)


if __name__ == '__main__':
    main(__file__, init=False)



# -*- coding: utf-8 -*-
"""
Created the 15/06/2023

@author: Sebastien Weber
"""
from typing import Union, List
import pyvisa

from pymeasure.instruments.thorlabs import thorlabs_elliptec as elliptec
from pymeasure.instruments.thorlabs.elliptec_utils.base import scan_for_devices
from pymeasure.instruments.resources import list_resources
from pymodaq.control_modules.move_utility_classes import DAQ_Move_base, comon_parameters_fun, main  # common set of parameters for all actuators
from pymodaq_utils.logger import set_logger, get_module_name
from pymodaq_utils.utils import ThreadCommand
from pymodaq_gui.parameter import Parameter

rm = pyvisa.ResourceManager()
com_ports = rm.list_resources()


class DAQ_Move_ElliptecPyMeasure(DAQ_Move_base):
    """ Plugin for the Elliptec Piezo driven motors from thorlabs

    This object inherits all functionality to communicate with PyMoDAQ Module through inheritance via DAQ_Move_base
    It then implements the particular communication with the instrument

    Attributes:
    -----------
    controller: object
        The particular object that allow the communication with the hardware, in general a python wrapper around the
         hardware library

    """
    _controller_units = '°'

    is_multiaxes = True
    axes_names = [str(ind) for ind in range(4)]
    _epsilon = 0.1

    params = [ {'title': 'COM port', 'name': 'com_port', 'type': 'list', 'limits': com_ports},
               {'title': 'Device', 'name': 'device', 'type': 'str'},
               ] + comon_parameters_fun(is_multiaxes, axes_names, epsilon=_epsilon)

    def ini_attributes(self):
        self.controller: elliptec.ElliptecController = None
        self.devices: List[str] = []

    def get_actuator_value(self):
        """Get the current value from the hardware with scaling conversion.

        Returns
        -------
        float: The position obtained after scaling conversion.
        """
        pos = self.controller.position
        pos = self.get_position_with_scaling(pos)
        return pos

    def close(self):
        """Terminate the communication protocol"""
        self.controller.close()

    def commit_settings(self, param: Parameter):
        """Apply the consequences of a change of value in the detector settings

        Parameters
        ----------
        param: Parameter
            A given parameter (within detector_settings) whose value has been changed by the user
        """
        ## TODO for your custom plugin
        if param.name() == "a_parameter_you've_added_in_self.params":
           self.controller.your_method_to_apply_this_param_change()
        else:
            pass

    def ini_stage(self, controller=None):
        """Actuator communication initialization

        Parameters
        ----------
        controller: (object)
            custom object of a PyMoDAQ plugin (Slave case). None if only one actuator by controller (Master case)

        Returns
        -------
        info: str
        initialized: bool
            False if initialization failed otherwise True
        """

        self.devices = scan_for_devices(self.settings['com_port'], start_address=0, stop_address=4)

        self.controller = self.ini_stage_init(old_controller=controller,
                                              new_controller=elliptec.ElliptecController(self.settings['com_port']))
        address = int(self.settings['multiaxes', 'axis'])
        device = f'Motor:{self.devices[address]["Motor Type"]} / '\
                 f'serial:{self.devices[address]["Serial No."]}'
        self.settings.child('device').setValue(device)


        info = device
        initialized = True
        return info, initialized

    def move_abs(self, value):
        """ Move the actuator to the absolute target defined by value

        Parameters
        ----------
        value: (float) value of the absolute target positioning
        """

        value = self.check_bound(value)  #if user checked bounds, the defined bounds are applied here
        self.target_value = value
        value = self.set_position_with_scaling(value)  # apply scaling if the user specified one

        self.controller.set_angle(value)  # when writing your own plugin replace this line

    def move_rel(self, value):
        """ Move the actuator to the relative target actuator value defined by value

        Parameters
        ----------
        value: (float) value of the relative target positioning
        """
        value = self.check_bound(self.current_position + value) - self.current_position
        self.target_value = value + self.current_position
        value = self.set_position_relative_with_scaling(value)

        self.controller.shift_angle(value)  # when writing your own plugin replace this line

    def move_home(self):
        """Call the reference method of the controller"""
        self.controller.home()  # when writing your own plugin replace this line
        self.emit_status(ThreadCommand('Update_Status', ['Some info you want to log']))

    def stop_motion(self):
        """Stop the actuator and emits move_done signal"""
        pass


if __name__ == '__main__':
    main(__file__, init=False)


